#include <stdio.h>
#include <ctype.h>
#include <stdlib.h>
#include <string.h>
#include "clustalw.h"


/*
 *   Prototypes
 */
sint local_penalty(sint penalty, sint n, sint *pweight);
void calc_p_penalties(ECHAR **aln, sint n, sint fs, sint ls, sint *weight);
/*
 *   Global variables
 */

extern sint nalphabet;
extern sint gap_dist;
extern sint max_aa;
extern sint GAPCOL,LENCOL;
extern sint debug;
extern Boolean use_endgaps;
extern Boolean endgappenalties;
extern Boolean no_pref_penalties;
extern DCHAR *alphabet;
extern char gappenfile[];

sint    *pas_op=NULL;

float reduced_gap = 0.3;
Boolean npref_pen;
sint gdist;                  /* local copy of gap_dist */

Boolean read_user_gappen(char *filename)
{
        sint i,j,err;
        int gap;
	char *activity;
	DCHAR c;
        static char line[MAXLINE+1];
	sint    *op=NULL;
	FILE *infile;

        if((infile=fopen(filename,"r"))==NULL) {
                warning("Cannot find penalty file [%s]\nSwitching off user defined penalties!",filename);
		no_pref_penalties=TRUE;
                return FALSE;
        }

	op = (sint *)ckalloc((max_aa+1) * sizeof(sint));
	activity = (char *)ckalloc((nalphabet+2) * sizeof(char));

        while(fgets(line,MAXLINE+1,infile) != NULL) {
                if(blankline(line)) continue;
		for(i=0;i<=nalphabet;i++) activity[i]='\0';
                if((err=sscanf(line,"%s%d",activity,&gap))!=2) {
                        warning("Bad format in penalty file [%s], %s: Switching off user defined penalties!",filename,line);
			op=ckfree(op);
			fclose(infile);
			no_pref_penalties=TRUE;
                        return FALSE;
                }
		if(strlen(activity)!=nalphabet) {
                        warning("Bad format in penalty file [%s], %s: Switching off user defined penalties!",filename,activity);
			op=ckfree(op);
			fclose(infile);
			no_pref_penalties=TRUE;
                        return FALSE;
                }

		for(j=0;j<nalphabet;j++) 
			c.c[j]=activity[j];
/* find activity in alphabet */
		for(i=0;i<max_aa;i++) {
			if(cmpchar(c,alphabet[i])) {
				op[i]=gap;
				break;
			}
		}
        }
	if(pas_op!=NULL) ckfree(pas_op);
	pas_op=op;
	if(activity!=NULL) ckfree(activity);
	fclose(infile);
	return TRUE;
}


void calc_gap_coeff(ECHAR **alignment, sint *gaps, sint **profile, Boolean struct_penalties,
         char *gap_penalty_mask, sint first_seq, sint last_seq,
         sint prf_length, sint gapcoef, sint lencoef)
{

   char c;
   sint i, j;
   sint is, ie;
   static sint numseq,val;
   static sint *gap_pos;
   static sint *p_weight;
   static float scale;
   
   numseq = last_seq - first_seq;

   for (j=0; j<prf_length; j++)
        gaps[j] = 0;
/*
   Check for a gap penalty mask
*/
   if (struct_penalties != NONE)
     {
	npref_pen = TRUE;
        gdist = 0;
     }
   else
     {
	npref_pen = no_pref_penalties;
        gdist = gap_dist;
     }                  
     
   if(no_pref_penalties==FALSE) {
	if (!read_user_gappen(gappenfile)) {
		npref_pen=no_pref_penalties=TRUE;
	}
   }

   for (i=first_seq; i<last_seq; i++)
     {
/*
   Include end gaps as gaps ?
*/
        is = 0;
        ie = prf_length;
        /*if (use_endgaps == FALSE && endgappenalties==FALSE)*/
        if (endgappenalties==FALSE)
        {
          for (j=0; j<prf_length; j++)
            {
              c = alignment[i][j];
              if ((c < 0) || (c >= max_aa))
                 is++;
              else
                 break;
            }
          for (j=prf_length-1; j>=0; j--)
            {
              c = alignment[i][j];
              if ((c < 0) || (c >= max_aa))
                 ie--;
              else
                 break;
            }
        }

        for (j=is; j<ie; j++)
          {
              if ((alignment[i][j] < 0) || (alignment[i][j] >= max_aa))
                 gaps[j]++;
          }
     }

   if (npref_pen == FALSE)
     {
        p_weight = (sint *) ckalloc( (prf_length+2) * sizeof (sint) );
        calc_p_penalties(alignment, prf_length, first_seq, last_seq, p_weight);
     }


   gap_pos = (sint *) ckalloc( (prf_length+2) * sizeof (sint) );
/*
    mark the residues close to an existing gap (set gaps[i] = -ve)
*/
   if (gdist <= 0)
     {
       for (i=0;i<prf_length;i++) gap_pos[i] = gaps[i];
     }
   else
     {
       i=0;
       while (i<prf_length)
         {
            if (gaps[i] <= 0)
              {
                 gap_pos[i] = gaps[i];
                 i++;
              }
            else 
              {
                 for (j = -gdist+1; j<0; j++)
                  {
                   if ((i+j>=0) && (i+j<prf_length) &&
                       ((gaps[i+j] == 0) || (gaps[i+j] < j))) gap_pos[i+j] = j;
                  }
                 while (gaps[i] > 0)
                    {
                       if (i>=prf_length) break;
                       gap_pos[i] = gaps[i];
                       i++;
                    }
                 for (j = 0; j<gdist; j++)
                  {
                   if (gaps[i+j] > 0) break;
                   if ((i+j>=0) && (i+j<prf_length) && 
                       ((gaps[i+j] == 0) || (gaps[i+j] < -j))) gap_pos[i+j] = -j-1;
                  }
                 i += j;
              }
         }
     }
if (debug>1)
{
fprintf(stdout,"gap open %d gap ext %d\n",(pint)gapcoef,(pint)lencoef);
fprintf(stdout,"gaps:\n");
  for(i=0;i<prf_length;i++) fprintf(stdout,"%d ", (pint)gaps[i]);
  fprintf(stdout,"\n");
fprintf(stdout,"gap_pos:\n");
  for(i=0;i<prf_length;i++) fprintf(stdout,"%d ", (pint)gap_pos[i]);
  fprintf(stdout,"\n");
}


   for (j=0;j<prf_length+1; j++)
     {
          
        if (gap_pos[j] <= 0)
          {
/*
    apply local gap penalties.
*/
              	profile[j+1][GAPCOL] = local_penalty(gapcoef, j, p_weight);
              	profile[j+1][LENCOL] = lencoef;

/*
    increase gap penalty near to existing gaps.
*/
             if (gap_pos[j] < 0)
                {
                    profile[j+1][GAPCOL] *= 2.0+2.0*(gdist+gap_pos[j])/gdist;
                }


          }
        else
          {
             scale = ((float)(numseq-gaps[j])/(float)numseq) * reduced_gap;
             profile[j+1][GAPCOL] = scale*gapcoef;
             profile[j+1][LENCOL] = 0.5 * lencoef;
          }
/*
    apply the gap penalty mask
*/
        if (struct_penalties != NONE)
          {
            val = gap_penalty_mask[j]-'0';
            if (val > 0 && val < 10)
              {
                profile[j+1][GAPCOL] *= val;
                profile[j+1][LENCOL] *= val;
              }
          }
/*
   make sure no penalty is zero - even for all-gap positions
*/
        if (profile[j+1][GAPCOL] <= 0) profile[j+1][GAPCOL] = 1;
        if (profile[j+1][LENCOL] <= 0) profile[j+1][LENCOL] = 1;
     }

/* set the penalties at the beginning and end of the profile */
   if(use_endgaps==TRUE || endgappenalties==TRUE)
     {
        profile[0][GAPCOL] = gapcoef;
        profile[0][LENCOL] = lencoef;
     }
   else
     {
        profile[0][GAPCOL] = 0;
        profile[0][LENCOL] = 0;
        profile[prf_length][GAPCOL] = 0;
        profile[prf_length][LENCOL] = 0;
     }
if (debug>1)
{
  fprintf(stdout,"Opening penalties:\n");
  for(i=0;i<=prf_length;i++) fprintf(stdout," %d:%d ",i, (pint)profile[i][GAPCOL]);
  fprintf(stdout,"\n");
}
if (debug>1)
{
  fprintf(stdout,"Extension penalties:\n");
  for(i=0;i<=prf_length;i++) fprintf(stdout,"%d:%d ",i, (pint)profile[i][LENCOL]);
  fprintf(stdout,"\n");
}

   if (npref_pen == FALSE)
        p_weight=ckfree((void *)p_weight);



   gap_pos=ckfree((void *)gap_pos);
}              
            

void calc_p_penalties(ECHAR **aln, sint n, sint fs, sint ls, sint *weight)
{
  char ix;
  sint j,k,numseq;
  sint i;

  numseq = ls - fs;
  for (i=0;i<n;i++)
    {
      weight[i] = 0;
      for (k=fs;k<ls;k++)
        {
                ix = aln[k][i];
                if ((ix < 0) || (ix >= max_aa)) continue;
                weight[i] += (180-pas_op[ix]);
                break;
        }
      weight[i] /= numseq;
    }

}

sint local_penalty(sint penalty, sint n, sint *pweight)
{

  Boolean h = FALSE;
  float gw;

  gw = 1.0;
  if (npref_pen == FALSE)
    {
       gw *= ((float)pweight[n]/100.0);
    }

  gw *= penalty;
  return((sint)gw);

}

